
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <ctype.h>

#include <GL/gl.h>

#include "defines.h"
#include "atofont_core.h"
#include "atofont_sb.h"


/* the buffers */
static float *tc = NULL, *ve = NULL;
static int nvertices = 0;



/* draw the characters one by one */
static int _atofont_string_buffer_write(struct atofont *font, unsigned char *s) {

  struct atocharacter *c;
  unsigned int i, k, n;
  float fg, m;


  if (font == NULL || s == NULL)
    return FAILED;

  /* compute the length of the string */
  i = 0;
  while (s[i] != 0)
    i++;

  /* adjust the buffers */
  if (nvertices < i) {
    tc = realloc(tc, sizeof(float) * i*6*2 * 2);
    ve = &tc[i*6*2];
    if (tc == NULL) {
      nvertices = 0;
      ve = NULL;
      fprintf(stderr, "_ATOFONT_STRING_BUFFER_WRITE: Out of memory error.\n");
      return FAILED;
    }
    nvertices = i;
  }

  i = 0;
  n = 0;
  m = 0.0f;
  fg = font->scale_x + font->spacing;

  while (s[i] != 0) {
    k = s[i++];
    c = &(font->characters[k]);

    if (font->type == ATOFONT_TYPE_PROPORTIONAL) {
      fg = c->left_margin*font->scale_x;
      m -= fg;
    }

    /* copy the vertices and texcoords */
    ve[n] = font->vertices[0*2 + 0] + m;
    tc[n] = c->x;
    n++;
    ve[n] = font->vertices[0*2 + 1];
    tc[n] = c->y + font->character_fixed_height;
    n++;

    ve[n] = font->vertices[1*2 + 0] + m;
    tc[n] = c->x + font->character_fixed_width;
    n++;
    ve[n] = font->vertices[1*2 + 1];
    tc[n] = c->y + font->character_fixed_height;
    n++;

    ve[n] = font->vertices[2*2 + 0] + m;
    tc[n] = c->x;
    n++;
    ve[n] = font->vertices[2*2 + 1];
    tc[n] = c->y;
    n++;

    ve[n] = font->vertices[1*2 + 0] + m;
    tc[n] = c->x + font->character_fixed_width;
    n++;
    ve[n] = font->vertices[1*2 + 1];
    tc[n] = c->y + font->character_fixed_height;
    n++;

    ve[n] = font->vertices[2*2 + 0] + m;
    tc[n] = c->x;
    n++;
    ve[n] = font->vertices[2*2 + 1];
    tc[n] = c->y;
    n++;

    ve[n] = font->vertices[3*2 + 0] + m;
    tc[n] = c->x + font->character_fixed_width;
    n++;
    ve[n] = font->vertices[3*2 + 1];
    tc[n] = c->y;
    n++;

    if (font->type == ATOFONT_TYPE_FIXED)
      m += fg;
    else
      m += fg + c->width*font->scale_x + font->spacing;
  }

  /* draw the string */
  glVertexPointer(2, GL_FLOAT, 0, ve);
  glTexCoordPointer(2, GL_FLOAT, 0, tc);
  glDrawArrays(GL_TRIANGLES, 0, n>>1);

  return SUCCEEDED;
}


int atofont_string_buffer_write_billboard(struct atofont *font, char *s, float x, float y, float z, int flags) {

  float p;


  if (font == NULL || s == NULL)
    return FAILED;

  /* fix the modelview */
  _atofont_billboard_spherical_cheat_apply(x, y, z);

  if ((flags & ATOFONT_CENTERED) != 0) {
    p = _atofont_get_string_width(font, (unsigned char *)s);
    glTranslatef(-p/2.0f, -font->scale_y/2.0f, 0);
  }

  /* write the string */
  _atofont_string_buffer_write(font, (unsigned char *)s);

  /* return the modelview */
  glPopMatrix();

  return SUCCEEDED;
}


int atofont_string_buffer_write_2D(struct atofont *font, char *s, float x, float y, int flags) {

  float p;


  if (font == NULL || s == NULL)
    return FAILED;

  glPushMatrix();

  if ((flags & ATOFONT_CENTERED) != 0) {
    p = _atofont_get_string_width(font, (unsigned char *)s);
    glTranslatef(-p/2.0f + x, -font->scale_y/2.0f + y, 0);
  }
  else
    glTranslatef(x, y, 0);

  /* write the string */
  _atofont_string_buffer_write(font, (unsigned char *)s);

  glPopMatrix();

  return SUCCEEDED;
}
