
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <time.h>

#include <SDL/SDL.h>
#include <SDL/SDL_video.h>
#include <SDL/SDL_keysym.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include "defines.h"
#include "opengl.h"
#include "../atofont_core.h"
#include "../atofont_so.h"
#include "../atofont_sb.h"
#include "../atofont_s.h"


/* example fonts */
extern struct atofont *ef_fixed, *ef_proportional;
extern struct atostringobject *so_fixed, *so_proportional;

/* zooming */
float zoom = 1.0;

/* texture id */
GLuint tid = 0;


int opengl_init(unsigned char *td, int tt, int tx, int ty) {

  glClearColor(0, 0, 0, 1);
  glShadeModel(GL_FLAT);
  glDepthMask(GL_FALSE);

  glDisable(GL_CULL_FACE);
  glDisable(GL_DEPTH_TEST);
  glEnable(GL_BLEND);
  glDisable(GL_LIGHTING);
  glDisable(GL_CULL_FACE);
  glDisable(GL_FOG);
  glDisable(GL_NORMALIZE);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  glEnableClientState(GL_VERTEX_ARRAY);
  glEnableClientState(GL_TEXTURE_COORD_ARRAY);

  glGenTextures(1, &tid);

  glBindTexture(GL_TEXTURE_2D, tid);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexImage2D(GL_TEXTURE_2D, 0, tt, tx, ty, 0, tt, GL_UNSIGNED_BYTE, td);

  return SUCCEEDED;
}


int opengl_exit(void) {

  return SUCCEEDED;
}


int opengl_render_scene(void) {

  GLfloat vertices[12*2];


  glClear(GL_COLOR_BUFFER_BIT);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(-1, 1, -1, 1, 0, 0);
 
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D, tid);

  /* write plain strings */
  glColor4f(1, 1, 1, 1);
  atofont_string_write_billboard(ef_fixed, "F: Billboard Centered", 0, 0, 0, ATOFONT_CENTERED);
  atofont_string_write_2D(ef_fixed, "F: 2D Centered", 0, 0.1, ATOFONT_CENTERED);
  atofont_string_write_2D(ef_fixed, "F: 2D", 0, -0.1, 0);
  atofont_string_write_billboard(ef_fixed, "F: Billboard", 0, -0.2, 0, 0);
  atofont_string_write_2D(ef_proportional, "P: 2D Centered", 0, 0.2, ATOFONT_CENTERED);
  atofont_string_write_2D(ef_proportional, "P: 2D", -0.1, -0.3, 0);
  atofont_string_write_billboard(ef_proportional, "P: Billboard Centered", 0.0, -0.4, 0, ATOFONT_CENTERED);
  atofont_string_write_billboard(ef_proportional, "P: Billboard", 0.0, 0.3, 0, 0);

  /* string objects */
  glColor4f(0.5, 0.6, 1, 1);
  atofont_string_object_write_2D(so_fixed, -0.5, -0.5, 0);
  atofont_string_object_write_2D(so_proportional, -0.5, -0.6, ATOFONT_CENTERED);

  /* bufferend string drawing */
  glColor4f(1.0, 0.6, 0.5, 1);
  atofont_string_buffer_write_billboard(ef_fixed, "F: Billboard Centered", 0, 0.4, 0, ATOFONT_CENTERED);
  atofont_string_buffer_write_2D(ef_proportional, "P: 2D Centered", 0, 0.5, ATOFONT_CENTERED);

  /* markers */
  glDisable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D, 0);

  glPointSize(5);
  glColor4f(0.8f, 0, 0, 1.0f);

  vertices[0*2+0] = 0.0;
  vertices[0*2+1] = 0.1;
  vertices[1*2+0] = 0.0;
  vertices[1*2+1] = 0.0;
  vertices[2*2+0] = 0.0;
  vertices[2*2+1] = -0.1;
  vertices[3*2+0] = 0.0;
  vertices[3*2+1] = -0.2;
  vertices[4*2+0] = -0.1;
  vertices[4*2+1] = -0.3;
  vertices[5*2+0] = 0.0;
  vertices[5*2+1] = -0.4;
  vertices[6*2+0] = 0.0;
  vertices[6*2+1] = 0.2;
  vertices[7*2+0] = 0.0;
  vertices[7*2+1] = 0.3;
  vertices[8*2+0] = -0.5;
  vertices[8*2+1] = -0.5;
  vertices[9*2+0] = -0.5;
  vertices[9*2+1] = -0.6;
  vertices[10*2+0] = 0;
  vertices[10*2+1] = 0.4;
  vertices[11*2+0] = 0;
  vertices[11*2+1] = 0.5;

  glVertexPointer(2, GL_FLOAT, 0, vertices);
  glDrawArrays(GL_POINTS, 0, 12);

  SDL_GL_SwapBuffers();

  return SUCCEEDED;
}
